extends CanvasLayer

@export var tracking : UltraleapHandTracking

@export var subtitle : Label
@export var subtitle_animator : AnimationPlayer

@export var tracking_choice : OptionButton

signal tracking_mode_request(tracking_mode : UltraleapTypes.TrackingMode)
signal device_selection(serial : String)

var current_device : UltraleapDevice

func _ready():
	subtitle_animator.play("show_subtitle")
	subtitle_animator.seek(0, true)
	subtitle_animator.stop(false)


func play_floating_bottom_text(text : String):
	subtitle.text = text
	subtitle_animator.play("show_subtitle")


func tracking_mode_changed(tracking_mode):
	tracking_choice.selected = tracking_mode
	play_floating_bottom_text("Tracking mode changed to " + tracking_mode_to_str(tracking_mode))


func button_item_selected(index):
	var error = emit_signal("tracking_mode_request", index)
	if error != OK:
		printerr("Failure emitting the signal for tracking mode request")
		
	current_device.set_tracking_mode(index)


func on_device_added(device):
	$Devices.add_item(device.serial)
	if current_device == null:
		current_device = device
		tracking_choice.selected = current_device.get_tracking_mode()


func on_device_removed(device):
	for i in range($Devices.item_count):
		if $Devices.get_item_text(i) == device.serial:
			$Devices.remove_item(i)
			break


func device_selected(index):
	var serial : String = $Devices.get_item_text(index)

	current_device = tracking.devices[index]
	tracking_choice.selected = current_device.get_tracking_mode()

	var error = emit_signal("device_selection", serial)
	if error != OK:
		printerr("Failure emitting the signal for device selection")


func tracking_mode_to_str(tracking_mode : UltraleapTypes.TrackingMode):
	return ["Desktop", "HMD", "Screentop"][tracking_mode]
