extends TextureRect
class_name ImagePuller

enum Side { Left, Right }

@export var side : ImagePuller.Side = Side.Left
@export var tracker : UltraleapHandTracking
@export var img : Image
@export var data_img : Image

@onready var bg_data : PackedByteArray = PackedByteArray()
@onready var data : PackedByteArray

var last_size : Vector2i = Vector2i(0, 0)

var device_index : int = 0
	
func _process(_delta):
	var view_size : Vector2i = get_viewport().size
	view_size.x /= 2
	
	var length = min(view_size.x, view_size.y)
	
	# Repaint on viewport size change
	if (view_size != last_size):
		_paint_black(view_size)
		last_size = view_size
		texture.set_image(img)

	if not Engine.is_editor_hint():
		if Input.is_action_just_pressed("next_device") and device_index < tracker.devices.size() - 1:
			device_index += 1
			
		if Input.is_action_just_pressed("previous_device") and device_index > 0:
			device_index -= 1

		if tracker.devices.size() == 0:
			return

		var image : UltraleapImage
		var device : UltraleapDevice = tracker.devices[device_index]
		
		if tracker.tracking_mode == UltraleapTypes.TrackingMode.Desktop:
			image = device.get_right_image() if side == Side.Left else device.get_left_image()
		else:
			image = device.get_right_image() if side == Side.Right else device.get_left_image()
			
		flip_h = tracker.tracking_mode == UltraleapTypes.TrackingMode.Desktop

		data = image.get_data()

		if data.size() == 384 * 384:
			data_img.set_data(384, 384, false, Image.FORMAT_L8, data)
			data_img.resize(length, length, Image.INTERPOLATE_NEAREST)

			var x : int = view_size.x - length if side == Side.Left else 0
			var y : int = (view_size.y - length) / 2
			
			img.blit_rect(data_img, Rect2i(0, 0, length, length), Vector2(x, y))

		texture.set_image(img)

func _paint_black(view_size):
	var _data_size = bg_data.resize((int)(view_size.x * view_size.y))
	bg_data.fill(1)

	img.set_data(view_size.x, view_size.y, false, Image.FORMAT_L8, bg_data)

func on_hud_device_selection(serial):
	var inc : int = 0
	for device in tracker.devices:
		if device.serial == serial:
			device_index = inc
			return
		inc += 1

func _ready():
	var view_size : Vector2i = get_viewport().size
	view_size.x /= 2
	
	_paint_black(view_size)
