extends Node3D

@export var tracking : UltraleapHandTracking
@export var camera : Camera3D
@export var hands : DebugHands
@export_node_path("CanvasLayer") var hud

var current_device : UltraleapDevice
var device_index : int

# Called when the node enters the scene tree for the first time.
func _ready():
	pass


# Called every frame. 'delta' is the elapsed time since the previous frame.
func _process(_delta):
	pass


func _on_device_selection(serial):
	if current_device != null:
		_close_device()
	current_device = _get_device_from_serial(serial)
	_open_device()
	
	move_camera_based_on_tracking_mode(current_device.get_tracking_mode())


func _get_device_from_serial(serial : String):
	for device in tracking.devices:
		if device.serial == serial:
			return device


func _open_device():
	current_device.open()
	current_device.subscribe()
	current_device.tracking_mode_changed.connect(_on_tracking_mode_changed)


func _close_device():
	current_device.unsubscribe()
	current_device.close()
	current_device.tracking_mode_changed.disconnect(_on_tracking_mode_changed)


func _on_hand_tracking_device_added(device):
	if current_device == null:
		current_device = device
		_open_device()


func _on_hand_tracking_device_removed(_device):
	if current_device == null:
		return
	_close_device()
	current_device = null


func _on_tracking_mode_changed(tracking_mode):
	move_camera_based_on_tracking_mode(tracking_mode)
	get_node(hud).tracking_mode_changed(tracking_mode)


func move_camera_based_on_tracking_mode(tracking_mode : UltraleapTypes.TrackingMode):
	if (tracking_mode == UltraleapTypes.TrackingMode.HMD):
		hands.position = Vector3(0, 0.4, 0.6)
		hands.rotation = Vector3(deg_to_rad(90), deg_to_rad(180), 0)
		camera.position = hands.position
		camera.rotation = Vector3.ZERO
	if (tracking_mode == UltraleapTypes.TrackingMode.Desktop):
		hands.position = Vector3.ZERO
		hands.rotation = Vector3.ZERO
		camera.position = Vector3(0, 0.4, 0.6)
		camera.rotation = Vector3(deg_to_rad(-30), 0, 0)
	if (tracking_mode == UltraleapTypes.TrackingMode.Screentop):
		hands.position = Vector3(0, 1, 0)
		hands.rotation = Vector3(deg_to_rad(180), 0, 0)
		camera.position = hands.position
		camera.rotation = Vector3(deg_to_rad(-90), deg_to_rad(180), 0)
