#include "device.h"

#include <godot_cpp/core/class_db.hpp>

#include <godot_cpp/classes/global_constants.hpp>
#include <godot_cpp/classes/ref.hpp>
#include <godot_cpp/classes/node3d.hpp>

#include <godot_cpp/variant/utility_functions.hpp>

#include <LeapC.h>

using namespace godot;

void UltraleapDevice::_bind_methods() {
    ClassDB::bind_method(D_METHOD("set_baseline", "baseline"), &UltraleapDevice::set_baseline);
    ClassDB::bind_method(D_METHOD("get_baseline"), &UltraleapDevice::get_baseline);

    ClassDB::bind_method(D_METHOD("set_serial", "serial"), &UltraleapDevice::set_serial);
    ClassDB::bind_method(D_METHOD("get_serial"), &UltraleapDevice::get_serial);

    ClassDB::bind_method(D_METHOD("set_id", "id"), &UltraleapDevice::set_id);
    ClassDB::bind_method(D_METHOD("get_id"), &UltraleapDevice::get_id);

    ClassDB::bind_method(D_METHOD("set_last_frame", "last_frame"), &UltraleapDevice::set_last_frame);
    ClassDB::bind_method(D_METHOD("get_last_frame"), &UltraleapDevice::get_last_frame);

    ClassDB::bind_method(D_METHOD("set_tracking_mode", "tracking_mode"), &UltraleapDevice::set_tracking_mode, DEFVAL(0));
    ClassDB::bind_method(D_METHOD("get_tracking_mode"), &UltraleapDevice::get_tracking_mode);

    ClassDB::bind_method(D_METHOD("get_left_image"), &UltraleapDevice::get_left_image);
    ClassDB::bind_method(D_METHOD("get_right_image"), &UltraleapDevice::get_right_image);

    ClassDB::bind_method(D_METHOD("open"), &UltraleapDevice::open);
    ClassDB::bind_method(D_METHOD("close"), &UltraleapDevice::close);
    ClassDB::bind_method(D_METHOD("subscribe"), &UltraleapDevice::subscribe);
    ClassDB::bind_method(D_METHOD("unsubscribe"), &UltraleapDevice::unsubscribe);

    ClassDB::add_property(
        "UltraleapDevice",
        PropertyInfo(
            Variant::INT,
            "baseline"
        ),
        "set_baseline",
        "get_baseline"
    );

    ClassDB::add_property(
        "UltraleapDevice",
        PropertyInfo(
            Variant::STRING,
            "serial"
        ),
        "set_serial",
        "get_serial"
    );

    ClassDB::add_property(
        "UltraleapDevice",
        PropertyInfo(
            Variant::INT,
            "id"
        ),
        "set_id",
        "get_id"
    );

    // Signals

    ClassDB::add_signal(
        "UltraleapDevice",
        MethodInfo(
            "tracking_mode_changed",
            PropertyInfo(
                Variant::INT,
                "tracking_mode",
                PROPERTY_HINT_ENUM,
                "Desktop, HMD, Screentop"
            )
        )
    );

    ClassDB::add_signal(
        "UltraleapDevice",
        MethodInfo(
            "frame_received",
            PropertyInfo(
                Variant::OBJECT,
                "frame",
                PROPERTY_HINT_RESOURCE_TYPE,
                "UltraleapFrame"
            )
        )
    );

    ClassDB::add_signal(
        "UltraleapDevice",
        MethodInfo(
            "image_received",
            PropertyInfo(
                Variant::OBJECT,
                "frame",
                PROPERTY_HINT_RESOURCE_TYPE,
                "UltraleapImage"
            )
        )
    );
}

void UltraleapDevice::set_tracking_mode(UltraleapTypes::TrackingMode value) {
    if (value != tracking_mode) {
        uint64_t set_flag = 0;
        uint64_t clear_flag = 0;
        if (value == UltraleapTypes::TrackingMode::Desktop) {
            clear_flag = eLeapPolicyFlag_OptimizeHMD | eLeapPolicyFlag_OptimizeScreenTop;
        }
        else if (value == UltraleapTypes::TrackingMode::HMD) {
            set_flag = eLeapPolicyFlag_OptimizeHMD;
            clear_flag = eLeapPolicyFlag_OptimizeScreenTop;
        }
        else if (value == UltraleapTypes::TrackingMode::Screentop) {
            clear_flag = eLeapPolicyFlag_OptimizeHMD;
            set_flag = eLeapPolicyFlag_OptimizeScreenTop;
        }
        else {
            UtilityFunctions::print("Invalid value for tracking mode");
        }
        LeapSetPolicyFlagsEx(connection, device, set_flag, clear_flag);
    }
}

void UltraleapDevice::tracking_mode_changed(UltraleapTypes::TrackingMode value) {
    emit_signal("tracking_mode_changed", value);
    tracking_mode = value;
}

void UltraleapDevice::subscribe() {
    LeapSubscribeEvents(connection, device);
}

void UltraleapDevice::unsubscribe() {
    LeapUnsubscribeEvents(connection, device);
}

void UltraleapDevice::open() {
    LeapOpenDevice(device_ref, &device);
}

void UltraleapDevice::close() {
    LeapCloseDevice(device);
}

void UltraleapDevice::on_frame_received(LEAP_TRACKING_EVENT* frame) {
    UltraleapFrame::fill_frame_data(last_frame, frame);
    emit_signal("frame_received", last_frame);
}

void UltraleapDevice::on_image_received(LEAP_IMAGE* image) {
    UltraleapImage::fill_images_data(left_image_ref, right_image_ref, image);
}