#ifndef ULTRALEAP_DEVICE_H
#define ULTRALEAP_DEVICE_H

#include <godot_cpp/classes/global_constants.hpp>
#include <godot_cpp/classes/ref.hpp>
#include <godot_cpp/classes/resource.hpp>

#include <godot_cpp/core/binder_common.hpp>

#include <godot_cpp/variant/utility_functions.hpp>

#include <LeapC.h>

#include "types.h"
#include "image.h"
#include "frame.h"

using namespace godot;

class UltraleapDevice : public Resource {
    GDCLASS(UltraleapDevice, Resource);

public:
    uint32_t baseline;
    String serial;
    uint32_t id;

    UltraleapTypes::TrackingMode tracking_mode = UltraleapTypes::TrackingMode::Desktop;

    UltraleapFrame* last_frame = memnew(UltraleapFrame);
    Ref<UltraleapFrame> last_frame_ref = Ref<UltraleapFrame>(last_frame);

    UltraleapImage* left_image = memnew(UltraleapImage);
    Ref<UltraleapImage> left_image_ref = Ref<UltraleapImage>(left_image);

    UltraleapImage* right_image = memnew(UltraleapImage);
    Ref<UltraleapImage> right_image_ref = Ref<UltraleapImage>(right_image);

    Variant get_last_frame() { return last_frame_ref; }
    void set_last_frame(Variant value) { return; }

    UltraleapTypes::TrackingMode get_tracking_mode() { return tracking_mode; }
    void set_tracking_mode(UltraleapTypes::TrackingMode value);

    Variant get_left_image() { return left_image_ref; }
    Variant get_right_image() { return right_image_ref; }

    uint32_t get_baseline() { return baseline; }
    void set_baseline(uint32_t value) { baseline = value; }

    String get_serial() { return serial; }
    void set_serial(String value) { serial = value; }

    uint32_t get_id() { return id; }
    void set_id(uint32_t value) { id = value; }

    void subscribe();
    void unsubscribe();

    void open();
    void close();

    LEAP_DEVICE_REF device_ref;
    LEAP_DEVICE device;
    LEAP_CONNECTION connection;

    void tracking_mode_changed(UltraleapTypes::TrackingMode value);

    void on_frame_received(LEAP_TRACKING_EVENT* frame);
    void on_image_received(LEAP_IMAGE* image);

protected:
    static void _bind_methods();
};


#endif