#ifndef ULTRALEAP_H
#define ULTRALEAP_H

#include <godot_cpp/classes/node3d.hpp>
#include <godot_cpp/classes/global_constants.hpp>
#include <godot_cpp/classes/viewport.hpp>
#include <godot_cpp/variant/vector3.hpp>
#include <godot_cpp/variant/quaternion.hpp>

#include <godot_cpp/core/binder_common.hpp>

#include <LeapC.h>
#include <thread>

#include "types.h"
#include "frame.h"
#include "image.h"
#include "device.h"
#include "device_list.h"

using namespace godot;

class UltraleapHandTracking : public Node3D {
    GDCLASS(UltraleapHandTracking, Node3D);

public:
    ~UltraleapHandTracking();

    void start();

    UltraleapTypes::TrackingMode tracking_mode = UltraleapTypes::TrackingMode::Desktop;

    bool interpolate = true;

    bool get_interpolate();
    void set_interpolate(bool value);

    LEAP_TRACKING_EVENT* latest_frame = nullptr;
    LEAP_IMAGE_EVENT* last_image_event = new LEAP_IMAGE_EVENT;

    UltraleapDeviceList* devices = memnew(UltraleapDeviceList);

    UltraleapFrame* last_frame = memnew(UltraleapFrame);
    Ref<UltraleapFrame> last_frame_ref = Ref<UltraleapFrame>(last_frame);

    UltraleapImage* left_image = memnew(UltraleapImage);
    Ref<UltraleapImage> left_image_ref = Ref<UltraleapImage>(left_image);

    UltraleapImage* right_image = memnew(UltraleapImage);
    Ref<UltraleapImage> right_image_ref = Ref<UltraleapImage>(right_image);

    Variant get_last_frame() { return last_frame_ref; }
    void set_last_frame(Variant value) { return; }

    UltraleapTypes::TrackingMode get_tracking_mode() { return tracking_mode; }
    void set_tracking_mode(UltraleapTypes::TrackingMode value);

    Variant get_left_image() { return left_image_ref; }
    Variant get_right_image() { return right_image_ref; }

    Array get_devices() { return devices->devices; }
    void set_devices(Array value) { devices->devices = value; }

protected:
    static void _bind_methods();

private:
    bool _isRunning = false;
    LEAP_CONNECTION connectionHandle = NULL;
    LEAP_CLOCK_REBASER clockSynchronizer;

    bool IsConnected = false;
    std::thread messageLoop;

    void handleConnectionEvent(const LEAP_CONNECTION_EVENT *evt);
    void serviceMessageLoop();

    void handle_device_event(const LEAP_DEVICE_EVENT* event);
    void handle_device_lost_event(const LEAP_DEVICE_EVENT* event);
    void handle_tracking_mode_event(const LEAP_TRACKING_MODE_EVENT* event, uint32_t device_id);

    void refresh_device_list();

    Ref<UltraleapDevice> create_device(LEAP_DEVICE_REF ref);
};

#endif